#!/usr/bin/perl
#
################################################################################
#
# IPCop Extra Interface CGI
#   Web Interface to configure Extra Interfaces on your IPCop
#
# Copyright (C) 2006-2014 Olaf (weizen_42) Westrik
#
# This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version. 
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
#
# You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA 
#
#
# Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der GNU General Public License, wie von der Free Software Foundation verffentlicht, weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder (nach Ihrer Option) jeder spteren Version. 
#
# Die Verffentlichung dieses Programms erfolgt in der Hoffnung, da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 
#
# Sie sollten ein Exemplar der GNU General Public License zusammen mit diesem Programm erhalten haben. Falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
#
################################################################################
#
# For support post / read in http://www.ipcop-forum.de
#
# $Id: xtiface.cgi 553 2014-05-28 12:01:15Z weizen_42 $
#
# Revision history:
#
# 2006-08-xx created by weizen_42 for xtiface AddOn
# 2006-12-22 4 interfaces, proxy access
# 2007-02    bugfix IP / network calculations
# 2007-04-02 transparent proxy, variable initialisation etc.
#


# Add entry in menu
# MENUENTRY network 089 "Extra Interfaces" "Extra Interfaces"

use strict;

# enable only the following on debugging purpose
use warnings;
use CGI::Carp 'fatalsToBrowser';
my $debug = 0;

use LWP::UserAgent;
use NetAddr::IP;

require '/usr/lib/ipcop/general-functions.pl';
require '/usr/lib/ipcop/lang.pl';
require '/usr/lib/ipcop/header.pl';

my $version = 'v1.0.2';

# checking for new version
my $addonname = 'xtiface';
my $onlineversion = '';
my $onlinelink = '';
my $timestamplastcheck = '/var/ipcop/xtiface/lastcheck';
my $flagdonotcheck = '/var/ipcop/xtiface/noversioncheck';

# get rid of used only once warnings
my @onlyonce = ( );
undef @onlyonce;

my $max_ifaces = 4;
my @ITFs=('GREEN','BLUE','ORANGE','RED');
my $counter;
my $ic;
my $i = 0;
my $j = 0;
my $line = '';
my $errormessage = '';


&Header::showhttpheaders();
my %hashcgi = ();
$hashcgi{'ACTION'} = '';
my %hashnet = ();
my %hashxtiface = ();

for $i ( 1 .. $max_ifaces ) {
    $hashcgi{"GRAY_${i}_DEV"}             = 'disabled';
    $hashcgi{"GRAY_${i}_ADDRESS"}         = "";
    $hashcgi{"GRAY_${i}_NETMASK"}         = "";
    $hashcgi{"GRAY_${i}_COPDNS"}          = "off";
    $hashcgi{"GRAY_${i}_COPPROXY"}        = 0;
    $hashcgi{"GRAY_${i}_TRANSPARENT"}     = "off";
    $hashcgi{"GRAY_${i}_REDHTTP"}         = "off";
    $hashcgi{"GRAY_${i}_REDDNS"}          = "off";
    $hashcgi{"GRAY_${i}_REDALL"}          = "off";
    $hashcgi{"GRAY_${i}_NOGREEN"}         = "off";
    $hashcgi{"GRAY_${i}_ROUTER"}          = "off";

    $hashxtiface{"GRAY_COUNT"}            = 0;
    $hashxtiface{"GRAY_${i}_DEV"}         = "disabled";
    $hashxtiface{"GRAY_${i}_ADDRESS"}     = "";
    $hashxtiface{"GRAY_${i}_NETMASK"}     = "";
    $hashxtiface{"GRAY_${i}_COPDNS"}      = "off";
    $hashxtiface{"GRAY_${i}_COPPROXY"}    = 0;
    $hashxtiface{"GRAY_${i}_TRANSPARENT"} = "off";
    $hashxtiface{"GRAY_${i}_REDHTTP"}     = "off";
    $hashxtiface{"GRAY_${i}_REDDNS"}      = "off";
    $hashxtiface{"GRAY_${i}_REDALL"}      = "off";
    $hashxtiface{"GRAY_${i}_NOGREEN"}     = "off";
    $hashxtiface{"GRAY_${i}_ROUTER"}      = "off";
}

&General::getcgihash(\%hashcgi);
$hashnet{'RED_1_ADDRESS'} = '';
$hashnet{'RED_1_NETMASK'} = '';
&General::readhash('/var/ipcop/ethernet/settings', \%hashnet);
&General::readhash('/var/ipcop/addons/xtiface/xtiface.conf', \%hashxtiface);

my $ifaces_count = 0;
my %ifaces_list = ();
my @gray_option = ("", "", "", "", "");
my @gray_checked_copdns = ("", "", "", "", "");
my @gray_checked_transparent = ("", "", "", "", "");
my @gray_checked_redhttp = ("", "", "", "", "");
my @gray_checked_reddns = ("", "", "", "", "");
my @gray_checked_redall = ("", "", "", "", "");
my @gray_checked_nogreen = ("", "", "", "", "");
my @gray_checked_router = ("", "", "", "", "");


if ( $hashcgi{'ACTION'} eq 'save' ) {
    for $i ( 1 .. $max_ifaces ) {
        if ( $hashcgi{"GRAY_${i}_DEV"} ne 'disabled' ) {
            for $j ( 1 .. $max_ifaces ) {
                next if ( $i == $j );

                if ( $hashcgi{"GRAY_${i}_DEV"} eq $hashcgi{"GRAY_${j}_DEV"} ) {
                    $errormessage = sprintf($Lang::tr{'xtiface error dup'}, $i, $j);
                    goto ERROREXIT;
                }
            }
            unless( &General::validip($hashcgi{"GRAY_${i}_ADDRESS"}) ) {
                $errormessage = $Lang::tr{'xtiface error ip'};
                goto ERROREXIT;
            }
            unless( &General::validip($hashcgi{"GRAY_${i}_NETMASK"}) ) {
                $errormessage = $Lang::tr{'xtiface error mask'};
                goto ERROREXIT;
            }

            my $ip = new NetAddr::IP($hashcgi{"GRAY_${i}_ADDRESS"}, $hashcgi{"GRAY_${i}_NETMASK"});

            if ( &General::IpInSubnet($hashcgi{"GRAY_${i}_ADDRESS"}, $hashnet{"GREEN_1_ADDRESS"}, $hashnet{"GREEN_1_NETMASK"}) ) {
                $errormessage = $Lang::tr{'xtiface error green'}." (".$hashcgi{"GRAY_${i}_ADDRESS"}.")";
                goto ERROREXIT;
            }
            if ( ($hashnet{"BLUE_COUNT"} > 0) && &General::IpInSubnet($hashcgi{"GRAY_${i}_ADDRESS"}, $hashnet{"BLUE_1_ADDRESS"}, $hashnet{"BLUE_1_NETMASK"}) ) {
                $errormessage = $Lang::tr{'xtiface error blue'}." (".$hashcgi{"GRAY_${i}_ADDRESS"}.")";
                goto ERROREXIT;
            }
            if ( ($hashnet{"ORANGE_COUNT"} > 0) && &General::IpInSubnet($hashcgi{"GRAY_${i}_ADDRESS"}, $hashnet{"ORANGE_1_ADDRESS"}, $hashnet{"ORANGE_1_NETMASK"}) ) {
                $errormessage = $Lang::tr{'xtiface error orange'}." (".$hashcgi{"GRAY_${i}_ADDRESS"}.")";
                goto ERROREXIT;
            }
            if ( $hashcgi{"GRAY_${i}_TRANSPARENT"} eq 'on' && $hashcgi{"GRAY_${i}_COPPROXY"} == 0 ) {
                $errormessage = $Lang::tr{'xtiface no proxyport'};
                goto ERROREXIT;
            }

            $hashxtiface{"GRAY_${i}_NETADDRESS"} = $ip->network()->addr();
            $hashxtiface{"GRAY_${i}_BROADCAST"} = $ip->broadcast()->addr();
            $hashxtiface{"GRAY_COUNT"} = $i;
        }

        if ( ($hashxtiface{"GRAY_${i}_DEV"} ne $hashcgi{"GRAY_${i}_DEV"}) && ($hashxtiface{"GRAY_${i}_DEV"} ne 'disabled') ) {
            # IFace was defined and has changed -> down
            system("/usr/local/bin/launch-xtiface --stop=$i >/dev/null 2>&1");
        }

        $hashxtiface{"GRAY_${i}_DEV"}        = $hashcgi{"GRAY_${i}_DEV"};
        $hashxtiface{"GRAY_${i}_ADDRESS"}    = $hashcgi{"GRAY_${i}_ADDRESS"};
        $hashxtiface{"GRAY_${i}_NETMASK"}    = $hashcgi{"GRAY_${i}_NETMASK"};
        $hashxtiface{"GRAY_${i}_COPDNS"}     = $hashcgi{"GRAY_${i}_COPDNS"};
        $hashxtiface{"GRAY_${i}_COPPROXY"}   = $hashcgi{"GRAY_${i}_COPPROXY"};
        $hashxtiface{"GRAY_${i}_TRANSPARENT"} = $hashcgi{"GRAY_${i}_TRANSPARENT"};
        $hashxtiface{"GRAY_${i}_REDHTTP"}    = $hashcgi{"GRAY_${i}_REDHTTP"};
        $hashxtiface{"GRAY_${i}_REDDNS"}     = $hashcgi{"GRAY_${i}_REDDNS"};
        $hashxtiface{"GRAY_${i}_REDALL"}     = $hashcgi{"GRAY_${i}_REDALL"};
        $hashxtiface{"GRAY_${i}_NOGREEN"}    = $hashcgi{"GRAY_${i}_NOGREEN"};
        $hashxtiface{"GRAY_${i}_ROUTER"}     = $hashcgi{"GRAY_${i}_ROUTER"};
    }

    &General::writehash('/var/ipcop/addons/xtiface/xtiface.conf', \%hashxtiface);

    for $i ( 1 .. $max_ifaces ) {
        if ( $hashxtiface{"GRAY_${i}_DEV"} ne 'disabled' ) {
            # IFace up
            if ( $debug == 0 ) {
                system("/usr/local/bin/launch-xtiface --start=$i >/dev/null 2>&1");
            }
            else {
                system("/usr/local/bin/launch-xtiface --start=$i ");
            }
        }
    }
    system("/usr/local/bin/restartdhcp");
}
ERROREXIT:

&Header::openpage($Lang::tr{'xtiface title'}, 1, '');

&Header::openbigbox('100%', 'left', '', $errormessage);

if ( $errormessage ) {
    &Header::openbox('100%', 'left', $Lang::tr{'error messages'}, 'error');
    print "<class name='base'>$errormessage\n";
    print "&nbsp;</class>\n";
    &Header::closebox();
}


# Found this usefull piece of code in BlockOutTraffic AddOn  8-)
#   fwrules.cgi
###############
# DEBUG DEBUG
if ( $debug ) {
    &Header::openbox('100%', 'left', 'DEBUG');
    my $debugCount = 0;
    foreach my $line (sort keys %hashcgi) {
        print "$line = $hashcgi{$line}<br />\n";
        $debugCount++;
    }
    print "&nbsp;Count: $debugCount\n";

    print "<hr />";
    foreach my $line (sort keys %hashxtiface) {
        if ( defined($hashxtiface{$line}) ) {
            print "$line = $hashxtiface{$line}<br />\n";
        }
        else {
            print "$line = <br />\n";
        }
    }
    &Header::closebox();
}
# DEBUG DEBUG
###############


#
# Check for new version
#
&checkfornewversion($addonname, $version);
if ($onlineversion ne '') {
    &Header::openbox('100%', 'left', $Lang::tr{'information messages'}, 'warning');
    print <<END
<table width="100%"><tr>
<td>$Lang::tr{'xtiface newversion'} <a href="$onlinelink" target="_blank"><b>$onlineversion</b></a></td>
</tr></table>
END
    ;
    &Header::closebox();
}


my @ifaces_list_all = `cat /proc/net/dev`;

foreach $line ( sort @ifaces_list_all )
{
    chomp($line);
    next unless ($line =~ /\s*(.*\d+):\s*/);

    my $device = $1;
    next if ($device =~ /ppp\d*/);
    next if ($device =~ /tun\d*/);

    if ($device eq $hashnet{'GREEN_1_DEV'}) {
        print "$device GREEN <br />" if ( $debug );
    }
    elsif (($hashnet{'RED_COUNT'} > 0) && ($device eq $hashnet{'RED_1_DEV'})) {
        print "$device RED <br />" if ( $debug );
    }
    elsif (($hashnet{'BLUE_COUNT'} > 0) && ($device eq $hashnet{'BLUE_1_DEV'})) {
        print "$device BLUE <br />" if ( $debug );
    }
    elsif (($hashnet{'ORANGE_COUNT'} > 0) && ($device eq $hashnet{'ORANGE_1_DEV'})) {
        print "$device ORANGE <br />" if ( $debug );
    }
    else {
        print "$device not used <br />"  if ( $debug );
        $ifaces_count++;

        for $i ( 1 .. $max_ifaces ) {
            if ( $hashxtiface{"GRAY_${i}_DEV"} eq $device ) {
                $gray_option[$i] = $gray_option[$i]."<option value='$device' selected='selected'>$device</option>";
            }
            else {
                $gray_option[$i] = $gray_option[$i]."<option value='$device'>$device</option>";
            }
        }
    }
}

for $i ( 1 .. $ifaces_count ) {
    if ( ($hashxtiface{"GRAY_${i}_DEV"} eq 'disabled') || ($hashxtiface{"GRAY_${i}_DEV"} eq 'disabled') ) {
        $gray_option[$i] = "<option value='disabled' selected='selected'>$Lang::tr{'xtiface disabled'}</option>".$gray_option[$i];
    }
    else {
        $gray_option[$i] = "<option value='disabled'>$Lang::tr{'xtiface disabled'}</option>".$gray_option[$i];
    }

    $gray_checked_copdns[$i] = "checked='checked'"      if ( $hashxtiface{"GRAY_${i}_COPDNS"} eq 'on' );
    $gray_checked_transparent[$i] = "checked='checked'" if ( $hashxtiface{"GRAY_${i}_TRANSPARENT"} eq 'on' );
    $gray_checked_redhttp[$i] = "checked='checked'"     if ( $hashxtiface{"GRAY_${i}_REDHTTP"} eq 'on' );
    $gray_checked_reddns[$i] = "checked='checked'"      if ( $hashxtiface{"GRAY_${i}_REDDNS"} eq 'on' );
    $gray_checked_redall[$i] = "checked='checked'"      if ( $hashxtiface{"GRAY_${i}_REDALL"} eq 'on' );
    $gray_checked_nogreen[$i] = "checked='checked'"     if ( $hashxtiface{"GRAY_${i}_NOGREEN"} eq 'on' );
    $gray_checked_router[$i] = "checked='checked'"      if ( $hashxtiface{"GRAY_${i}_ROUTER"} eq 'on' );
}

&Header::openbox('100%', 'left', $Lang::tr{'xtiface title'});

print "<form method='post' action='$ENV{'SCRIPT_NAME'}'><table width='100%'>";

# show standard interfaces for reference
foreach my $interface (@ITFs) {
    for ($counter = 1; $counter <= $hashnet{"${interface}_COUNT"}; $counter++) {
        $ic = "${interface}_${counter}";
        my $lc_itf=lc($interface);
        my $colorname = "${lc_itf}";

        print <<END
<tr><td width='25%' class='boldbase'><b><font color='$colorname'>$Lang::tr{"$lc_itf interface"}</font></b></td><td>&nbsp;</td></tr>
END
;
        print <<END
<tr><td width='25%'>$Lang::tr{'device'}:</td><td><b>$hashnet{"${ic}_DEV"}</b></td></tr>
END
;
        print <<END
<tr><td width='25%'>$Lang::tr{'ip address'}/$Lang::tr{'netmask'}:</td><td>$hashnet{"${ic}_ADDRESS"}&nbsp;/&nbsp;$hashnet{"${ic}_NETMASK"}</td></tr>
END
;
    }
}

print "</table><hr /><table width='100%'>";

for $i ( 1 .. $max_ifaces ) {
    if ( $ifaces_count >= $i ) {
        print <<END
<tr><td width='25%' class='boldbase'><b>$Lang::tr{"gray_${i} interface"}</b></td><td colspan='3'>&nbsp;</td></tr>
<tr><td width='25%'>$Lang::tr{'device'}:</td><td colspan='3'><select name='GRAY_${i}_DEV'>$gray_option[$i]</select></td></tr>
<tr><td width='25%'>$Lang::tr{'ip address'}:</td><td width='25%'><input type='text' name='GRAY_${i}_ADDRESS' value='$hashxtiface{"GRAY_${i}_ADDRESS"}' size='18' /></td>
    <td width='25%'>$Lang::tr{'netmask'}:</td><td width='25%'><input type='text' name='GRAY_${i}_NETMASK' value='$hashxtiface{"GRAY_${i}_NETMASK"}' size='18' /></td></tr>
<!--
<tr><td width='30%' class='base'>$Lang::tr{'xtiface COPDNS'}:</td><td colspan='3'><input type='checkbox' name='GRAY_${i}_COPDNS' $gray_checked_copdns[$i] /></td></tr>
<tr><td width='30%' class='base'>$Lang::tr{'xtiface COPPROXY'}:&nbsp;<img src='/blob.gif' alt='*' /></td><td><input type='text' name='GRAY_${i}_COPPROXY' value='$hashxtiface{"GRAY_${i}_COPPROXY"}' size='5'/></td>
    <td width='30%' class='base'>$Lang::tr{'xtiface TRANSPARENT'}:</td><td width='20%'><input type='checkbox' name='GRAY_${i}_TRANSPARENT' $gray_checked_transparent[$i] /></td></tr>
<tr><td width='30%' class='base'>$Lang::tr{'xtiface REDHTTP'}:</td><td colspan='3'><input type='checkbox' name='GRAY_${i}_REDHTTP' $gray_checked_redhttp[$i] /></td></tr>
<tr><td width='30%' class='base'>$Lang::tr{'xtiface REDDNS'}:</td><td colspan='3'><input type='checkbox' name='GRAY_${i}_REDDNS' $gray_checked_reddns[$i] /></td></tr>
<tr><td width='30%' class='base'>$Lang::tr{'xtiface REDALL'}:</td><td colspan='3'><input type='checkbox' name='GRAY_${i}_REDALL' $gray_checked_redall[$i] /></td></tr>
<tr><td width='30%' class='base'>$Lang::tr{'xtiface NOGREEN'}:</td><td colspan='3'><input type='checkbox' name='GRAY_${i}_NOGREEN' $gray_checked_nogreen[$i] /></td></tr>
<tr><td width='30%' class='base'>$Lang::tr{'xtiface ROUTER'}:</td><td colspan='3'><input type='checkbox' name='GRAY_${i}_ROUTER' $gray_checked_router[$i] /></td></tr>
-->
END
;
    }
}

print <<END
</table><hr /><table width='100%'>
<tr>
    <td class='comment1button'>&nbsp;</td>
    <td class='button1button'><input type='submit' value='$Lang::tr{'save'}' /><input type='hidden' name='ACTION' value='save' /></td>
    <td class='onlinehelp'>
        <a href='http://www.ban-solms.de/t/IPCop-xtiface.html' target='_blank'>
        <img src='/images/web-support.png' alt='$Lang::tr{'online help en'}' title='$Lang::tr{'online help en'}' /></a>
    </td>
</tr>
</table>
<hr />
<table width='100%'>
<tr>
    <td class='comment1button'>&nbsp;</td>
    <td colspan='2' align='right'>
        <b><small><a href="http://www.ban-solms.de/t/IPCop.html" target="_blank">XTIFACE $version</a></small></b>
    </td>
</tr>
</table>
</form>
END
;
&Header::closebox();

&Header::closebigbox();
&Header::closepage();


sub checkfornewversion
{
  my $addon = shift;
  my $version = shift;
  $onlineversion = '';

  if ( -e $flagdonotcheck )
  {
    return;
  }

  # only check if we are online and last check was some time ago
  if ( (! -e '/var/ipcop/red/active') || (-e $timestamplastcheck) && (int(-M $timestamplastcheck) < 5) )
  {
    return;
  }

#workaround to suppress a warning when a variable is used only once
  my @dummy = ( $General::version );
  undef (@dummy);

  my $ua = LWP::UserAgent->new;
  $ua->timeout(120);
  $ua->agent("Mozilla/4.0 (compatible; IPCop $General::version; $version)");
  my $content = $ua->get("http://www.ban-solms.de/version/$addon");

  if ( $content->is_success )
  {
    # compare the versions, format is v1.2.3
    $content->content =~ /v(\d+).(\d+).(\d+)/;
    my $ver1 = $1;
    my $ver2 = $2;
    my $ver3 = $3;

    $version =~ /v(\d+).(\d+).(\d+)/;

    if ( ($ver1 > $1) || (($ver1 == $1) && ($ver2 > $2)) || (($ver1 == $1) && ($ver2 == $2) && ($ver3 > $3)) )
    {
      $onlineversion = "v$ver1.$ver2.$ver3";

      $content->content =~ /http(.*)/;
      $onlinelink = "http$1";
    }
    else
    {
      # no news, recheck in a couple of days
      system("touch $timestamplastcheck >/dev/null 2>&1");
    }
  }
}
